local Levels = GLOBAL.require("map/levels")

GLOBAL.require "playerprofile"
GLOBAL.require "playerdeaths"
GLOBAL.require "saveindex"
GLOBAL.require "screens/mainscreen"
GLOBAL.require "components/resurrectable"

local DeathScreen = GLOBAL.require "screens/deathscreen"

GLOBAL.require "screens/popupdialog"
GLOBAL.require "screens/bigpopupdialog"
GLOBAL.require "screens/endgamedialog"
GLOBAL.require "screens/deathscreen"


AddGlobalClassPostConstruct("saveindex", "SaveIndex", function(self)
	
	function self:StartForgottenForge(cb)
		local function ongamesaved()
			self.data.slots[self.current_slot].current_mode = "forgottenforge"
			self.data.slots[self.current_slot].modes.forgottenforge = {}
			self.data.slots[self.current_slot].modes.forgottenforge.playerdata = GLOBAL.GetPlayer():GetSaveRecord().data


			-- Hunt through the survival levels and find ours, then save its index.)
			--local Levels = require("map/levels")
			for i,level in ipairs(Levels.custom_levels) do
				if level.id == "FORGOTTENFORGE" then
					self.data.slots[self.current_slot].modes.forgottenforge.options = {
						level_id = i
					}
					print("Found a forgottenforge level:",i)
					break
				end
			end
			self:Save(cb)
		end

		self:SaveCurrent(ongamesaved)
		
	end

	function self:StopForgottenForge(cb)
		local filename = self.data.slots[self.current_slot].modes.forgottenforge.file

		local function onsavedindex()
			GLOBAL.EraseFiles(cb, {filename})
		end
		self.data.slots[self.current_slot].current_mode = "cave"
		self.data.slots[self.current_slot].modes.forgottenforge = {}
		self.data.slots[self.current_slot].modes.cave.playerdata = GLOBAL.GetPlayer():GetSaveRecord().data
		self:Save(onsavedindex)
	end

end)


--Add temperature when do anything

local function LavaHeatNightmare(inst,data)
	
	inst.components.temperature:SetTemperature(inst.components.temperature.current + 4)

end


local function HandleDeathCleanup2(player, data)
    local game_time = GLOBAL.GetClock():ToMetricsString()

    if GLOBAL.SaveGameIndex:GetCurrentMode() == "forgottenforge" then
	    local playtime = GLOBAL.GetTimePlaying()
	    playtime = math.floor(playtime*1000)
	    GLOBAL.SetTimingStat("time", "scenario", playtime)
	    GLOBAL.SendTrackingStats()
	    local days_survived, start_xp, reward_xp, new_xp, capped = GLOBAL.CalculatePlayerRewards(player)
	    
	    GLOBAL.ProfileStatsSet("xp_gain", reward_xp)
	    GLOBAL.ProfileStatsSet("xp_total", new_xp)
	    GLOBAL.SubmitCompletedLevel() --close off the instance

	    player.components.health.invincible = true

	    player.profile:Save(function()
		    GLOBAL.SaveGameIndex:EraseCurrent(function() 
				    GLOBAL.scheduler:ExecuteInTime(3, function() 
						TheFrontEnd:PushScreen(DeathScreen(days_survived, start_xp, nil, capped))
					end)
		    	end)
		    end)

		
	end
end

local function OnPlayerDeath2(player, data)
	print ("OnPlayerDeath")
	
	local cause = data.cause or "unknown"
	local will_resurrect = player.components.resurrectable and player.components.resurrectable:CanResurrect() 

	print ("OnPlayerDeath() ", cause, tostring(will_resurrect))
    player.HUD:Hide()
	
	if cause ~= "file_load" then
	    GLOBAL.TheMixer:PushMix("death")

		GLOBAL.Morgue:OnDeath({killed_by=cause, 
						days_survived=GLOBAL.GetClock().numcycles or 0,
						character=GLOBAL.GetPlayer().prefab, 
						location= (player.components.area_aware and player.components.area_aware.current_area.story) or "unknown", 
						world= (GLOBAL.GetWorld().meta and GLOBAL.GetWorld().meta.level_id) or "unknown"})

	    local game_time = GLOBAL.GetClock():ToMetricsString()
	    
		GLOBAL.RecordDeathStats(cause, GLOBAL.GetClock():GetPhase(), player.components.sanity.current, player.components.hunger.current, will_resurrect)

		GLOBAL.ProfileStatsAdd("killed_by_"..cause)
	    GLOBAL.ProfileStatsAdd("deaths")
	end

	if will_resurrect or cause == "file_load" then

		local res = player.components.resurrectable:FindClosestResurrector()
		print ("OnPlayerDeath() ", tostring(res))

		local delay = 4
		if cause == "file_load" then
			TheFrontEnd:Fade(false, 0)
			delay = 0
		end       
    else
		HandleDeathCleanup2(player, data)
    end
end

AddSimPostInit(function(player) 
	if player and GLOBAL.SaveGameIndex:GetCurrentMode() == "forgottenforge" then
		player:ListenForEvent("working",LavaHeatNightmare)
		player:ListenForEvent( "death", function(inst, data) OnPlayerDeath2(player, data) end)
	end
end)		

local function LavaVaveOff(component)
	local original_UpdateAmbientVolumes = component.UpdateAmbientVolumes
	component.UpdateAmbientVolumes = function(self, ...)
		
	if GLOBAL.SaveGameIndex:GetCurrentMode() == "forgottenforge" then

	--local is_winter = GetSeasonManager():IsWinter()
	
	local player = GLOBAL.GetPlayer()
	local sanity_level = 1
	if player.components.sanity ~= nil then
		sanity_level = player.components.sanity:GetPercent()
	end

	self.inst.SoundEmitter:SetParameter( "SANITY", "sanity", 1-sanity_level )
	

	for k,v in pairs(self.playing_sounds) do
		local vol = self.ambient_vol * v.volume
		
		if vol > 0 ~= v.playing then
			if vol > 0 then
				self.inst.SoundEmitter:PlaySound( v.sound, v.sound)
				self.inst.SoundEmitter:SetParameter( v.sound, "daytime", self.daynightparam )
			else
				self.inst.SoundEmitter:KillSound(v.sound)
			end
			v.playing = vol > 0
		end
		
		if v.playing then
			self.inst.SoundEmitter:SetVolume(v.sound, vol)
		end
	end
	
	if self.num_waves > 0 then
		
		if self.playing_waves then
			self.inst.SoundEmitter:KillSound("waves")
			self.playing_waves = false
		end
		
	else
		self.wave_volume = 0
		if self.playing_waves then
			self.inst.SoundEmitter:KillSound("waves")
			self.playing_waves = false
		end
	end

	end		
			original_UpdateAmbientVolumes(self, ...)
	end
end
AddComponentPostInit("ambientsoundmixer", LavaVaveOff)