local brain = require("brains/frogbrain")

local assets =
{
    Asset("ANIM", "anim/lavae.zip"),
  --  Asset("SOUND", "sound/together.fsb"),
}

local prefabs =
{
    "lavae_move_fx",
}

SetSharedLootTable( 'lavae_lava',
{
    {'houndfire',   1.0},
    {'houndfire',   1.0},
    {'houndfire',   1.0},
})

SetSharedLootTable( 'lavae_frozen',
{
    {'monstermeat',   1.0},
    {'rocks',       0.5},
    {'rocks',       0.4},
    {'rocks',       0.3},
})

local function OnCollide(inst, other)
    if other ~= nil and other:IsValid() and other.components.burnable ~= nil 
    and not other.components.fueled then
        other.components.burnable:Ignite(true, inst)
    end
end

local function RetargetFn(inst)
    if not inst.components.health:IsDead() and not inst.components.sleeper:IsAsleep() then
        return FindEntity(inst, TUNING.FROG_TARGET_DIST, function(guy) 
            if guy.components.combat and guy.components.health and not guy.components.health:IsDead() then
                return guy.components.inventory ~= nil
            end
        end)
    end
end


local function OnTargetDeath(inst, data)
    local new_target = inst.components.grouptargeter:SelectTarget()
    if new_target then
        inst.components.combat:SetTarget(new_target)
    end
end

local function OnNewTarget(inst, data)
    local old = data.oldtarget
    if old and old.lavae_ontargetdeathfn then 
        inst:RemoveEventCallback("death", old.lavae_ontargetdeathfn, old)
    end

    local new = data.target
    if new then
        new.lavae_ontargetdeathfn = function() OnTargetDeath(inst) end
        if new:HasTag("player") then
            inst:ListenForEvent("death", new.lavae_ontargetdeathfn, new)
        end
    end
end

--[[local function OnEntitySleep(inst)
    if inst.reset then
        inst:Remove()
    end
end]]

local function ShouldSleep(inst)
	return false 
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddDynamicShadow()
    inst.entity:AddLight()

    inst.DynamicShadow:SetSize(2, 1)
    inst.Transform:SetTwoFaced()
    MakeCharacterPhysics(inst, 50, 0.5)

    inst.AnimState:SetBank("lavae")
    inst.AnimState:SetBuild("lavae")
    inst.AnimState:PlayAnimation("idle")

    inst:AddTag("lavae")
    inst:AddTag("monster")
    inst:AddTag("hostile")

    inst.Light:SetRadius(2)
    inst.Light:SetFalloff(0.5)
    inst.Light:SetIntensity(0.75)
    inst.Light:SetColour(235/255, 121/255, 12/255)
    inst.Light:Enable(true)

    inst.Physics:SetCollisionCallback(OnCollide)

	inst:AddComponent("locomotor")
    inst.components.locomotor.walkspeed = 5.5

	inst:SetBrain(brain)
	inst:SetStateGraph("SGforgelavae")
	
	inst:AddComponent("sleeper")
	inst.components.sleeper:SetSleepTest(ShouldSleep)
	
	
    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(500)
    inst.components.health.fire_damage_scale = 0

	inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(50)
    inst.components.combat:SetRange(6, 3)
    inst.components.combat:SetAttackPeriod(4)
 --   inst.components.combat:SetKeepTargetFunction(KeepTargetFn)
    inst.components.combat:SetRetargetFunction(3, RetargetFn)

	inst:AddComponent("lootdropper")
    
	inst:AddComponent("knownlocations")
    inst:AddComponent("inspectable")


 --   MakeHauntablePanic(inst)

    MakeLargeFreezableCharacter(inst)
	inst.components.freezable:SetResistance(-10)

    return inst
end

STRINGS.NAMES.FORGELAVAE = "Lavae"

STRINGS.CHARACTERS.GENERIC.DESCRIBE.FORGELAVAE = {	
	"Too hot to handle.",
}

STRINGS.CHARACTERS.WX78.DESCRIBE.FORGELAVAE = {	
	"SMELLS LIKE BURNING",
	 
}

return Prefab("common/monsters/forgelavae", fn, assets, prefabs)