local BigPopupDialogScreen = require "screens/popupdialog"
require "prefabutil"

local assets=
{
	Asset("ANIM", "anim/ancient5.zip"),
	Asset("ATLAS", "images/inventoryimages/ancient5_map.xml"),
    Asset("IMAGE", "images/inventoryimages/ancient5_map.tex"),
	Asset("SOUND", "sound/common.fsb"),

}

local prefabs = 

{
	"trinket_6",
	"metal",
	"gears",
	"greengem",
	"yellowgem",
	"orangegem",
}

SetSharedLootTable( 'ancientloot5',
{
    {'trinket_6',  0.5},
	{'metal',  0.4},
	{'gears',  0.2},
	{'greengem',  0.02},
	{'yellowgem',  0.02},
	{'orangegem',  0.02},
 
})

local function GetVerb(inst)
	return "Travel Through"
end

local function ItemTradeTest(inst, item)
	if item:HasTag("forgekey") then
		return true
	end
	return false
end

local function ItemGet(inst, giver, item)
		inst.AnimState:PlayAnimation("activate")
		inst.AnimState:PushAnimation("idle_on", true)
		inst.SoundEmitter:PlaySound("dontstarve/common/teleportato/teleportato_addpart", "teleportato_addpart")
		inst.components.activatable.inactive = true
	
end

local function OnActivate_GoTo(inst)
	SetPause(true,"portal")
	
	local function dofight()
		local function onsaved()
		    StartNextInstance({reset_action=RESET_ACTION.LOAD_SLOT, save_slot = SaveGameIndex:GetCurrentSaveSlot()}, true)
		end
		TheFrontEnd:PopScreen()
		SetPause(false)
		GetPlayer().sg:GoToState("teleportato_teleport")
		SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition())
		inst.SoundEmitter:PlaySound("dontstarve/common/destroy_metal")
		SpawnPrefab("ancient5").Transform:SetPosition(inst.Transform:GetWorldPosition())
		inst:Remove()
		
		GetPlayer():DoTaskInTime(5, function() SaveGameIndex:StartForgottenForge(onsaved) end)
	end

	local function rejectfight()
		TheFrontEnd:PopScreen()
		SetPause(false) 
		inst.components.activatable.inactive = true
	end

	TheFrontEnd:PushScreen(BigPopupDialogScreen("You want go to the forge?", "Remember! It's a very hot place.",
			{{text="Yeah!", cb = dofight},
			 {text="Nah.", cb = rejectfight} }))
end

local function OnActivate_Return(inst)
	SetPause(true,"portal")
	
	local function gohome()
		local function onsaved()
				StartNextInstance({reset_action=RESET_ACTION.LOAD_SLOT, 
				save_slot = SaveGameIndex:GetCurrentSaveSlot(), 
				playeranim="wakeup"})
		end
		TheFrontEnd:PopScreen()
		SetPause(false)
		GetPlayer().sg:GoToState("teleportato_teleport")
		GetPlayer():DoTaskInTime(5, function() SaveGameIndex:StopForgottenForge(onsaved) end)
		
		           
	end

	local function stayhere()
		TheFrontEnd:PopScreen()
		SetPause(false) 
		inst.components.activatable.inactive = true
	end

	TheFrontEnd:PushScreen(BigPopupDialogScreen("Leave?", "Remember! You won't be able to come back here.",
			{{text="Yeah!", cb = gohome},
			 {text="Nah.", cb = stayhere} }))
		
end

local function onhammered(inst, worker)
	inst.components.lootdropper:DropLoot()
	SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_metal")
	inst:Remove()
end

local function onhit(inst, worker)
	inst.AnimState:PlayAnimation("idle")
	--inst.AnimState:PushAnimation("idle")
end


local function fn(Sim)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	
	local minimap = inst.entity:AddMiniMapEntity()
	minimap:SetIcon( "ancient5_map.tex" )
    
    inst:AddTag("structure")
	MakeObstaclePhysics(inst, .5)
    
    inst.AnimState:SetBank("ancient5")
    inst.AnimState:SetBuild("ancient5")
		
    inst:AddComponent("inspectable")
	
	
	inst:AddComponent("lootdropper")
	inst.components.lootdropper:SetChanceLootTable('ancientloot5')
	
    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
    inst.components.workable:SetWorkLeft(4)
	inst.components.workable:SetOnFinishCallback(onhammered)
	inst.components.workable:SetOnWorkCallback(onhit) 
	
	inst:AddComponent("inspectable")

	inst:AddComponent("activatable")
    inst.components.activatable.getverb = GetVerb
	inst.components.activatable.quickaction = true

	inst.SoundEmitter:PlaySound("shadwell_sfx/examples/portal", "portal_loop")
	
    return inst
end

local function travelportal_fn()
    local inst = fn()
	inst.components.activatable.inactive = false
    inst.components.activatable.OnActivate = OnActivate_GoTo
	
	inst.AnimState:PlayAnimation("idle")

	inst:AddComponent("trader")
	inst.components.trader:SetAcceptTest(ItemTradeTest)
	inst.components.trader.onaccept = ItemGet
	
    return inst
end

local function returnportal_fn()
    local inst = fn()
	
	inst.AnimState:PlayAnimation("idle_on")
    inst.AnimState:PushAnimation("idle_on", true)
	
	inst.components.activatable.inactive = true
    inst.components.activatable.OnActivate = OnActivate_Return	
    return inst
end

STRINGS.NAMES.FFPORTAL = "Gear Table"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.FFPORTAL = {	
	"What I will discover now?",
	"Table?",
	"I can put something on it",
	 
}

STRINGS.CHARACTERS.WX78.DESCRIBE.FFPORTAL = {	
	"INTERDIMENSIONAL PASSAGE DETECTED",
	 
}

STRINGS.CHARACTERS.WAXWELL.DESCRIBE.FFPORTAL = {	
	"It leads to my forgotten ideas",
	 
}

STRINGS.NAMES.FFPORTALRETURN = "Gear Table"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.FFPORTALRETURN = {	
	"Can I return?",
	 
}

STRINGS.CHARACTERS.WX78.DESCRIBE.FFPORTALRETURN = {	
	"INTERDIMENSIONAL PASSAGE DETECTED",
	 
}

STRINGS.CHARACTERS.WAXWELL.DESCRIBE.FFPORTALRETURN = {	
	"It's a way back",
	 
}

return Prefab( "common/ffportal", travelportal_fn, assets),
Prefab("common/ffportalreturn", returnportal_fn, assets)