local assets=
{
	Asset("ANIM", "anim/lapple_tree.zip"),
	
	Asset("ATLAS", "images/inventoryimages/lappletree.xml"),	
	Asset("IMAGE", "images/inventoryimages/lappletree.tex"),		
}


local prefabs = {"lapple_tree","charcoal","log","twigs",}    

local function getstatus(inst)
	
	if inst.components.pickable and not inst.components.pickable:CanBePicked() then
		STRINGS.CHARACTERS.GENERIC.DESCRIBE.LAPPLE_TREE = {	
		"I hope they will grow back...",
		}
		STRINGS.CHARACTERS.WX78.DESCRIBE.LAPPLE_TREE = {	
		"PATHETIC AND EMPTY TREE",
			 
		}
	elseif inst.components.burnable and inst.components.burnable:IsBurning() then
		STRINGS.CHARACTERS.GENERIC.DESCRIBE.LAPPLE_TREE = {	
		"That's burning fast!",
		"Burn baby burn!",
		}
		STRINGS.CHARACTERS.WX78.DESCRIBE.LAPPLE_TREE = {	
			"SMELLS LIKE BURNING",
			 
		}
	 elseif inst:HasTag("burnt") then
       	STRINGS.CHARACTERS.GENERIC.DESCRIBE.LAPPLE_TREE = {	
			"Burned by its inner fire...",
		}
		STRINGS.CHARACTERS.WX78.DESCRIBE.LAPPLE_TREE = {	
			"HA HA, DESTROYED",
			 
		}
	elseif inst:HasTag("stump") then
        STRINGS.CHARACTERS.GENERIC.DESCRIBE.LAPPLE_TREE = {	
			"Take that...fire nature!",
		}
		STRINGS.CHARACTERS.WX78.DESCRIBE.LAPPLE_TREE = {	
			"THE NATURAL WORLD IS OVERRATED",
			 
		}		
	else
	STRINGS.CHARACTERS.GENERIC.DESCRIBE.LAPPLE_TREE = {	
		"Hot and sweet.",
		"They look ripe. Time to harvest."
		}
		STRINGS.CHARACTERS.WX78.DESCRIBE.LAPPLE_TREE = {	
			"LIFE FINDS A WAY, UNFORTUNATELY",
			 
		}
	
	end
end  


local function onregenfn(inst)
	inst.AnimState:PlayAnimation("grow") 
	inst.AnimState:PushAnimation("idle_loop", true)
	inst.AnimState:Show("BANANA") 
end

local function makefullfn(inst)
	inst.AnimState:PlayAnimation("idle_loop", true)
	inst.AnimState:Show("BANANA") 
end


local function onpickedfn(inst)
	inst.SoundEmitter:PlaySound("dontstarve/wilson/pickup_reeds") 
	inst.AnimState:PlayAnimation("pick") 
	inst.AnimState:PushAnimation("idle_loop") 
	inst.AnimState:Hide("BANANA") 
end

local function makeemptyfn(inst)
	inst.AnimState:PlayAnimation("idle_loop")
	inst.AnimState:Hide("BANANA") 
end

local function dug(inst)
	inst:Remove()
	inst.components.lootdropper:SpawnLootPrefab("log")
end

local function setupstump(inst)
	inst.stump = true
	inst:RemoveComponent("pickable")
	inst.components.workable:SetWorkAction(ACTIONS.DIG)
    inst.components.workable:SetWorkLeft(1)
    inst.components.workable:SetOnWorkCallback(dug)
	inst.AnimState:PlayAnimation("idle_stump")
	inst:AddTag("stump")
end

local function chopped(inst)
	if inst.stump then
		return
	end
	inst.SoundEmitter:PlaySound("dontstarve/wilson/use_axe_tree")

	if inst.burnt then
		inst:RemoveComponent("workable")
		inst.SoundEmitter:PlaySound("dontstarve/forest/treeCrumble")
		inst.components.lootdropper:SpawnLootPrefab("charcoal")
		inst.persists = false
		inst.AnimState:PlayAnimation("chop_burnt")
		inst:DoTaskInTime(50*FRAMES, function() inst:Remove() end)
	else
	              
		inst.SoundEmitter:PlaySound("dontstarve/forest/treefall")

		inst.components.lootdropper:SpawnLootPrefab("log")
		inst.components.lootdropper:SpawnLootPrefab("twigs")
		inst.components.lootdropper:SpawnLootPrefab("twigs")
		inst.AnimState:Hide("BANANA") 
		if inst.components.pickable and inst.components.pickable.canbepicked then
			inst.components.lootdropper:SpawnLootPrefab("cacao")
		end
		setupstump(inst)
		inst.AnimState:PlayAnimation("fall")
		inst.AnimState:PushAnimation("idle_stump")
	end

end

local function chop(inst)
	inst.AnimState:PlayAnimation("chop")
	inst.AnimState:PushAnimation("idle_loop", true)
    inst.SoundEmitter:PlaySound("dontstarve/wilson/use_axe_tree")          

end

local function startburn(inst)
	inst.burnt = true
    if inst.components.pickable then
        inst:RemoveComponent("pickable")
    end
    
end


local function makeburnt(inst)
	inst.burnt = true
	
	inst:RemoveComponent("burnable")
	inst:RemoveComponent("propagator")
	inst:RemoveComponent("pickable")

	if inst.stump then
		inst:Remove()
	else
		inst.AnimState:PlayAnimation("burnt")
		inst.SoundEmitter:PlaySound("dontstarve/forest/treeCrumble")
    	inst.components.workable:SetWorkAction(ACTIONS.CHOP)
    	inst.components.workable:SetWorkLeft(1)
    end
	inst:AddTag("burnt")
end


local function onsave(inst, data)
	data.stump = inst.stump
	data.burnt = inst.burnt
end

local function onload(inst, data)
	if data and data.stump then
		setupstump(inst)
	elseif data and data.burnt then
		makeburnt(inst)
	end
end


local function fn(Sim)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
    local sound = inst.entity:AddSoundEmitter()
	MakeObstaclePhysics(inst,.5)

	local minimap = inst.entity:AddMiniMapEntity()
	minimap:SetIcon( "lappletree.tex" )	

    anim:SetBank("cave_banana_tree")
    anim:SetBuild("lapple_tree")
    anim:PlayAnimation("idle_loop",true)
    anim:SetTime(math.random()*2)

	inst:AddComponent("pickable")
	inst.components.pickable.picksound = "dontstarve/wilson/pickup_reeds"
	
	inst.components.pickable:SetUp("lapple", TUNING.CAVE_BANANA_GROW_TIME*1.5)  --- 6 days
	inst.components.pickable.onregenfn = onregenfn
	inst.components.pickable.onpickedfn = onpickedfn
	inst.components.pickable.makeemptyfn = makeemptyfn
	inst.components.pickable.makefullfn = makefullfn


	inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.CHOP)
    inst.components.workable:SetWorkLeft(3)
    inst.components.workable:SetOnFinishCallback(chopped)
    inst.components.workable:SetOnWorkCallback(chop)


	inst:AddComponent("lootdropper")
	
    inst:AddComponent("inspectable")    
	inst.components.inspectable.getstatus = getstatus
    ---------------------        
    MakeMediumBurnable(inst)
    MakeSmallPropagator(inst)
	MakeNoGrowInWinter(inst)    
    ---------------------   
    inst.components.burnable:SetOnIgniteFn(startburn)
	inst.components.burnable:SetOnBurntFn(makeburnt)
    inst.OnSave = onsave
    inst.OnLoad = onload
  
    return inst
end

STRINGS.NAMES.LAPPLE_TREE = "Fire Apple Tree"

return Prefab("cave/lapple_tree", fn, assets, prefabs)
