local assets=
{
	Asset("ANIM", "anim/metal_spark.zip"),
}

local sounds =
{
    takeoff = "dontstarve/creatures/mosquito/mosquito_takeoff",
    attack = "dontstarve/creatures/mosquito/mosquito_attack",
    buzz = "dontstarve/creatures/mosquito/mosquito_fly_LP",
    hit = "dontstarve/creatures/mosquito/mosquito_hurt",
    death = "dontstarve/creatures/mosquito/mosquito_death",
	explode = "dontstarve/creatures/mosquito/mosquito_explo",
}

SetSharedLootTable( 'metal_spark',
{
    {'trinket_6', .5},
})

local SHARE_TARGET_DIST = 30
local MAX_TARGET_SHARES = 10


local function SwapBelly(inst, size)
	for i=1,4 do
		if i == size then
			inst.AnimState:Show("body_"..tostring(i))
		else
			inst.AnimState:Hide("body_"..tostring(i))
		end
	end
end

local function TakeDrink(inst, data)
	inst.drinks = inst.drinks + 1
	if inst.drinks > inst.maxdrinks then
		inst.toofat = true
		inst.components.health:Kill()
	else
		SwapBelly(inst, inst.drinks)
	end
end

local function Retarget(inst)
	return GetPlayer()
end

local function KeepTarget(inst, target)
	return target == GetPlayer()
end


local function KillerRetarget(inst)
    return FindEntity(inst, 20, function(guy)
        return (guy:HasTag("character") or guy:HasTag("animal") or guy:HasTag("monster") )
            and not guy:HasTag("chess")
            and inst.components.combat:CanTarget(guy)
    end)
end


local function OnAttacked(inst, data)
    local attacker = data and data.attacker
    if attacker and attacker:HasTag("chess") then return end
    inst.components.combat:SetTarget(attacker)
    inst.components.combat:ShareTarget(attacker, SHARE_TARGET_DIST, function(dude) return dude:HasTag("chess") end, MAX_TARGET_SHARES)
end

local function mosquito()
	local inst = CreateEntity()

    inst.entity:AddTransform()
	inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	inst.entity:AddLightWatcher()
	inst.entity:AddDynamicShadow()
	inst.DynamicShadow:SetSize( .8, .5 )
    inst.Transform:SetFourFaced()


    ----------
	
	inst:AddTag("monster")
    inst:AddTag("hostile")
    inst:AddTag("chess")
	
	if IsDLCEnabled(CAPY_DLC) or IsDLCEnabled(PORKLAND_DLC) then 
		MakeAmphibiousCharacterPhysics(inst, 1, .5)
		inst.Physics:SetCollisionGroup(COLLISION.FLYERS)
		inst.Physics:CollidesWith(COLLISION.FLYERS)	
	else
		MakeCharacterPhysics(inst, 1, .5)
		inst.Physics:SetCollisionGroup(COLLISION.FLYERS)
		inst.Physics:ClearCollisionMask()
		inst.Physics:CollidesWith(COLLISION.WORLD)
		inst.Physics:CollidesWith(COLLISION.FLYERS)
	end

    inst.AnimState:SetBank("metal_spark")
    inst.AnimState:SetBuild("metal_spark")
    inst.AnimState:PlayAnimation("idle", true)
    inst.AnimState:SetRayTestOnBB(true);

    inst:AddComponent("locomotor") -- locomotor must be constructed before the stategraph
    inst.components.locomotor:EnableGroundSpeedMultiplier(false)
	inst.components.locomotor:SetTriggersCreep(false)
	inst.components.locomotor.walkspeed = TUNING.MOSQUITO_WALKSPEED
	inst.components.locomotor.runspeed = TUNING.MOSQUITO_RUNSPEED
	
	
	local brain = require "brains/nightmarecreaturebrain"
    inst:SetBrain(brain)
	
    inst:SetStateGraph("SGmechalance")

	inst.sounds = sounds 

	 ------------------

    ------------------
    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(10)

    ------------------
    inst:AddComponent("combat")
    inst.components.combat.hiteffectsymbol = "body"
    inst.components.combat:SetDefaultDamage(4)
	inst.components.combat:SetRange(TUNING.BAT_ATTACK_DIST)
    inst.components.combat:SetRetargetFunction(2, Retarget)
    inst.components.combat:SetKeepTargetFunction(KeepTarget)
	inst.components.combat:SetAttackPeriod(TUNING.MOSQUITO_ATTACK_PERIOD)

	inst.drinks = 1
	inst.maxdrinks = 1
	inst:ListenForEvent("onattackother", TakeDrink)
	SwapBelly(inst, 1)


    ------------------
    inst:AddComponent("knownlocations")

    ------------------

    inst:ListenForEvent("attacked", OnAttacked)

    return inst
end

STRINGS.NAMES.METAL_SPARK = "Ball Lightning"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.METAL_SPARK = {	
	"Natural phenomenon.",
}

STRINGS.CHARACTERS.WX78.DESCRIBE.METAL_SPARK = {	
	"TECHNOLOGICAL PHENOMENON.",
}

return Prefab( "forest/monsters/metal_spark", mosquito, assets) 
